--
--  AutoBar
--
--  Config functions
-- 
--  Maintained by Azethoth / Toadkiller of Proudmoore.  Original author Saien of Hyjal
--  http://www.curse-gaming.com/en/wow/addons-4430-1-autobar-toadkiller.html
--

AutoBarConfig = {};


StaticPopupDialogs["AUTOBAR_ITEMENTRY"] = {
	text = AUTOBAR_CONFIG_CUSTOM_ENTRY,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1, 
	maxLetters = 50,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		local text = editBox:GetText();
		if (tonumber(text) and tonumber(text) > 0) then
			text = tonumber(text);
		end
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
		if (type(tmp) == "table") then
			AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting][AutoBar_Options_ConfigButton_ChooseCategory.editting] = text;
		else
			AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting] = text;
		end
		AutoBar_Options_ConfigButton_ChooseCategory:Hide();
	end,
	OnShow = function()
		local editBox = getglobal(this:GetName().."WideEditBox");
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
		local txt;
		if (type(tmp) == "table") then
			txt = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting][AutoBar_Options_ConfigButton_ChooseCategory.editting];
		else
			txt = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
		end
		if (txt) then
			editBox:SetText(txt);
		else
			editBox:SetText("");
		end
		editBox:SetFocus();
	end,
	OnHide = function()
		local editBox = getglobal(this:GetName().."WideEditBox");
		if (ChatFrameEditBox:IsVisible()) then 
			ChatFrameEditBox:SetFocus();
		end
		editBox:SetText("");
	end,
	OnCancel = function()
		this:GetParent():Hide();
	end,
	EditBoxOnEnterPressed = function ()
		StaticPopupDialogs["AUTOBAR_ITEMENTRY"].OnAccept();
		this:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function ()
		this:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};


function AutoBarConfig_Toggle(msg)
	if (AutoBar_Options:IsVisible()) then
		AutoBar_Options:Hide();
	else
		AutoBar_Options:Show();
	end
end


function AutoBarConfig.OnLoad()
	SLASH_AUTOBAR1 = "/autobar";
	SLASH_AUTOBAR2 = "/atb";
	SlashCmdList["AUTOBAR"] = function (msg)
	    -- no damn keyboard commands, just toggle config dialog
		AutoBarConfig_Toggle(msg);
	end
end


function AutoBarConfig.OnShow()
	local index, button, hotkey;
	for index = 1, AUTOBAR_MAXBUTTONS, 1 do
		button = getglobal("AutoBar_Options_Buttons_Button"..index);
		hotkey = getglobal("AutoBar_Options_Buttons_Button"..index.."HotKey");
		count  = getglobal("AutoBar_Options_Buttons_Button"..index.."Count");
		icon   = getglobal("AutoBar_Options_Buttons_Button"..index.."Icon");
		hotkey:SetText("#"..index);
		if (AutoBar_Config[AutoBar_Player].buttons[index]) then
			local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[index];
			icon:SetTexture(AutoBar_GetTexture(buttonInfo));
			count:SetText("");
		else
			count:SetText(AUTOBAR_CONFIG_EMPTY);
			icon:SetTexture("");
		end
	end
	AutoBar_Options_Bar_Sliders();
	AutoBar_Options_CheckBox_Setup();
	AutoBarConfigSlot:Setup();
end


function AutoBar_Config_ButtonSetTooltip()
	local buttonInfo;
	local preamble, extended;
	if (this.itemid) then
		local name,itemid = GetItemInfo(this.itemid);
		if (name and itemid) then
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink(itemid);
		else
			local tmp = "item:"..this.itemid..":0:0:0";
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink("item:"..this.itemid..":0:0:0");
		end
		return;
	end
	if (this.category) then
		buttonInfo = this.category;
		extended=true;
	elseif (string.find(this:GetName(), "^AutoBar_Options_Buttons_Button")) then
		buttonInfo = AutoBar_Config[AutoBar_Player].buttons[this:GetID()];
		extended=true;
		preamble=true;
	elseif (string.find(this:GetName(), "^AutoBar_Options_ConfigButton_Button")) then
		extended=true;
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
		if (type(tmp) == "table") then
			buttonInfo = tmp[this:GetID()]
		else
			buttonInfo = tmp;
		end
	end
	local message = "";
	if (buttonInfo == "EMPTY") then
	elseif (type(buttonInfo) == "table") then
		if (preamble) then message = AUTOBAR_TOOLTIP9; end
		local index,cat;
		for index,cat in buttonInfo do
			if (type(cat) == "string" and AutoBar_Category_Info[cat]) then
				message = message.."\n"..AutoBar_Category_Info[cat].description;
			elseif (type(cat) == "number") then
				local name = GetItemInfo(cat);
				if (name) then
					message = message.."\n"..name..AUTOBAR_TOOLTIP10;
				else
					message = message..AUTOBAR_TOOLTIP11;
				end
			else
				message = message.."\n"..cat..AUTOBAR_TOOLTIP12;
			end
		end
	elseif (type(buttonInfo) == "string" and AutoBar_Category_Info[buttonInfo]) then
		if (preamble) then message = AUTOBAR_TOOLTIP13; end
		message = message..AutoBar_Category_Info[buttonInfo].description;
		if (extended) then
			message = message.."\n";
			if (AutoBar_Category_Info[buttonInfo].notusable) then
				message = message..AUTOBAR_TOOLTIP14;
			end
			if (AutoBar_Category_Info[buttonInfo].targetted) then
				if (AutoBar_Category_Info[buttonInfo].targetted == "WEAPON") then
					message = message..AUTOBAR_TOOLTIP15;
				else
					message = message..AUTOBAR_TOOLTIP16;
				end
			end
			if (AutoBar_Category_Info[buttonInfo].noncombat) then
				message = message..AUTOBAR_TOOLTIP17;
			end
			if (AutoBar_Category_Info[buttonInfo].combatonly) then
				message = message..AUTOBAR_TOOLTIP18;
			end
			if (AutoBar_Category_Info[buttonInfo].location) then
				message = message..AUTOBAR_TOOLTIP19..AutoBar_Category_Info[buttonInfo].location..".";
			end
			if (AutoBar_Category_Info[buttonInfo].limit) then
				message = message..AUTOBAR_TOOLTIP20;
				if (AutoBar_Category_Info[buttonInfo].limit.downhp) then
					message = message..AUTOBAR_TOOLTIP21;
					if (AutoBar_Category_Info[buttonInfo].limit.downmana) then
						message = message..", ";
					end
				end
				if (AutoBar_Category_Info[buttonInfo].limit.downmana) then
					message = message..AUTOBAR_TOOLTIP22;
				end
			end

		end
	elseif (type(buttonInfo) == "string" and not AutoBar_Category_Info[buttonInfo]) then
		if (preamble) then message = AUTOBAR_TOOLTIP23; end
		message = message..buttonInfo..AUTOBAR_TOOLTIP12;
	elseif (type(buttonInfo) == "number") then
		if (preamble) then message = AUTOBAR_TOOLTIP23; end
		local name = GetItemInfo(buttonInfo);
		if (name) then
			message = message..name..AUTOBAR_TOOLTIP10;
		else
			message = message..AUTOBAR_TOOLTIP11;
		end
	end

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	GameTooltip:SetText(message);
end


function AutoBar_Config_Button_Edit(slotIndex)
	if (not slotIndex) then slotIndex = AutoBar_Options_ConfigButton.editting; end
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[slotIndex];
	AutoBar_Options_ConfigButton.editting = slotIndex;
	AutoBar_Options_ConfigButton:Show();
	AutoBar_Options_ConfigButtonTitleText:SetText(AUTOBAR_CONFIG_EDITBUTTON.." #"..slotIndex);
	local noPopupCheckbox = getglobal("AutoBar_Options_ConfigButton_NoPopup");
	local arrangeOnUseCheckbox = getglobal("AutoBar_Options_ConfigButton_ArrangeOnUse");
--	local noPopupCheckboxText = getglobal("AutoBar_Options_ConfigButton_NoPopupText");
--	local arrangeOnUseCheckboxText = getglobal("AutoBar_Options_ConfigButton_ArrangeOnUseText");
	local index,tmp,i;

	if (type(buttonInfo) == "table") then
		noPopupCheckbox:Enable();
		arrangeOnUseCheckbox:Enable();
--		noPopupCheckboxText:Enable();
--		arrangeOnUseCheckboxText:Enable();
		noPopupCheckbox:SetChecked(buttonInfo.noPopup);
		arrangeOnUseCheckbox:SetChecked(buttonInfo.arrangeOnUse);
		tmp = 0;
		for index = 1, AUTOBAR_MAXSLOTCATEGORIES, 1 do
			if (AutoBar_Config[AutoBar_Player].buttons[slotIndex][index]) then
				tmp = index;
			end
		end
		index = 1;
		while (index < tmp) do 
			if (AutoBar_Config[AutoBar_Player].buttons[slotIndex][index]) then
				index = index + 1;
			else
				AutoBar_Config[AutoBar_Player].buttons[slotIndex][index] = 
				AutoBar_Config[AutoBar_Player].buttons[slotIndex][index+1];
				AutoBar_Config[AutoBar_Player].buttons[slotIndex][index+1] = nil;
				tmp = 0;
				for i = 1, AUTOBAR_MAXSLOTCATEGORIES, 1 do
					if (AutoBar_Config[AutoBar_Player].buttons[slotIndex][i]) then
						tmp = i;
					end
				end
			end
		end
	else
		noPopupCheckbox:Disable();
		arrangeOnUseCheckbox:Disable();
--		noPopupCheckboxText:Disable();
--		arrangeOnUseCheckboxText:Disable();
	end

	for index = 1, AUTOBAR_MAXSLOTCATEGORIES, 1 do
		local button   = getglobal("AutoBar_Options_ConfigButton_Button"..index);
		local hotkey   = getglobal("AutoBar_Options_ConfigButton_Button"..index.."HotKey");
		local count    = getglobal("AutoBar_Options_ConfigButton_Button"..index.."Count");
		local icon     = getglobal("AutoBar_Options_ConfigButton_Button"..index.."Icon");
		local move     = getglobal("AutoBar_Options_ConfigButton_Move"..index);
		local moveup   = getglobal("AutoBar_Options_ConfigButton_Move"..index.."_Up");
		local movedown = getglobal("AutoBar_Options_ConfigButton_Move"..index.."_Down");
		local checkbox = getglobal("AutoBar_Options_ConfigButton_Option"..index);
		local checkboxtext = getglobal("AutoBar_Options_ConfigButton_Option"..index.."Text");

		hotkey:Hide();
		if (type(buttonInfo) == "table") then
			if (buttonInfo[index]) then
				move:Show();
				count:SetText("");
				if (AutoBar_Category_Info[buttonInfo[index]] and AutoBar_Category_Info[buttonInfo[index]].targetted) then
					checkbox:Show();
					checkboxtext:SetText(AUTOBAR_CONFIG_SMARTSELFCAST);
					if (AutoBar_Config[AutoBar_Player].smartselfcast and AutoBar_Config[AutoBar_Player].smartselfcast[buttonInfo[index]]) then
						checkbox:SetChecked(1);
					else
						checkbox:SetChecked(0);
					end
				else
					checkbox:Hide();
				end
				if (index == 1) then
					moveup:Hide();
				else
					moveup:Show();
				end
				if (buttonInfo[index+1]) then
					movedown:Show();
				else
					movedown:Hide();
				end
			else
				move:Hide();
				count:SetText("Empty");
				checkbox:Hide();
			end
			icon:SetTexture(AutoBar_GetTexture(buttonInfo[index]));
			button:Show();
		else
			move:Hide();
			if (index == 1) then
				icon:SetTexture(AutoBar_GetTexture(buttonInfo));
				button:Show();
				if (buttonInfo) then
					count:SetText("");
					if (AutoBar_Category_Info[buttonInfo] and AutoBar_Category_Info[buttonInfo].targetted) then
						checkbox:Show();
						checkboxtext:SetText(AUTOBAR_CONFIG_SMARTSELFCAST);
						if (AutoBar_Config[AutoBar_Player].smartselfcast and AutoBar_Config[AutoBar_Player].smartselfcast[buttonInfo]) then
							checkbox:SetChecked(1);
						else
							checkbox:SetChecked(0);
						end
					else
						checkbox:Hide();
					end
				else
					count:SetText(AUTOBAR_CONFIG_EMPTY);
					checkbox:Hide();
				end
			else
				button:Hide();
				checkbox:Hide();
			end
		end
	end
	---
	if (type(buttonInfo) == "table") then
		AutoBar_Options_ConfigButton_ConvertButton:SetText(AUTOBAR_CONFIG_CONVERTBUTTON1);
	else
		AutoBar_Options_ConfigButton_ConvertButton:SetText(AUTOBAR_CONFIG_CONVERTBUTTON2);
	end
end


-- Hook PickupContainerItem so we know what bag & slot is being dragged from 
AutoBarConfig.PickupContainerItem = PickupContainerItem;
local dragBag = nil;
local dragSlot = nil;

function PickupContainerItem(bag, slot, ...)
	local returnTab = { AutoBarConfig.PickupContainerItem(bag, slot, unpack(arg)) };
	
	if (CursorHasItem()) then
		dragBag = bag;
		dragSlot = slot;
	end
	
	return unpack(returnTab);
end


local function AutoBar_ConfigButton_OnClick(mousebutton)
	if (CursorHasItem() and dragBag and dragSlot) then
        AutoBar_Config_Button_OnReceiveDrag();
	else
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
		if (type(tmp) == "table") then
			AutoBar_Options_ConfigButton_ChooseCategory.editting = this:GetID();
		else
			AutoBar_Options_ConfigButton_ChooseCategory.editting = nil;
		end
		AutoBar_Options_ConfigButton_ChooseCategory:Show();
	end
end


local function AutoBar_ConfigButton_ChooseCategory_OnClick(mousebutton)
	if (IsShiftKeyDown()) then
		if (AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore) then
			AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore = nil;
			this:GetParent():Hide();
		else
			local category = this.category;
			if (category == "EMPTY") then category = nil; end
			if (category) then
				AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore = category;
				AutoBar_Config_ButtonChooseCategory_OnShow();
			end
		end
	else
		local category = this.category;
		if (category == "EMPTY") then category = nil; end
		if (AutoBar_Options_ConfigButton_ChooseCategory.editting) then
			AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting][AutoBar_Options_ConfigButton_ChooseCategory.editting] = category;
		else
			AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting] = category;
		end
		AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore = nil;
		this:GetParent():Hide();
	end
end
	

function AutoBar_Config_Button_OnClick(mousebutton)
	ResetCursor();
	AutoBar_Options_ConfigButton.dragging = nil;
	AutoBar_Options_ConfigButton.draggingConfigButton = nil;
	if (string.find(this:GetName(), "^AutoBar_Options_Buttons_Button")) then
		return AutoBar_Config_Button_Edit(this:GetID());
	elseif (string.find(this:GetName(), "^AutoBar_Options_ConfigButton_Button")) then
		return AutoBar_ConfigButton_OnClick(mousebutton);
	elseif (string.find(this:GetName(), "^AutoBar_Options_ConfigButton_ChooseCategory_Button")) then
		return AutoBar_ConfigButton_ChooseCategory_OnClick(mousebutton);
	end
end


function AutoBarConfig.ButtonOnDragStart()
	local fromIndex = this:GetID();
	if (string.find(this:GetName(), "^AutoBar_Options_Buttons_Button")) then
		AutoBar_Options_ConfigButton.dragging = fromIndex;
	elseif (string.find(this:GetName(), "^AutoBar_Options_ConfigButton_Button")) then
		AutoBar_Options_ConfigButton.draggingConfigButton = fromIndex;
	end
	SetCursor("BUY_CURSOR");
end


function AutoBarConfig.ButtonOnReceiveDrag()
	local toIndex = this:GetID();
	if (string.find(this:GetName(), "^AutoBar_Options_Buttons_Button")) then
		local fromIndex = AutoBar_Options_ConfigButton.dragging;
		if (fromIndex and fromIndex ~= toIndex) then
			local fromButton = AutoBar_Config[AutoBar_Player].buttons[fromIndex];
			AutoBar_Config[AutoBar_Player].buttons[fromIndex] = AutoBar_Config[AutoBar_Player].buttons[toIndex];
			AutoBar_Config[AutoBar_Player].buttons[toIndex] = fromButton;
		end
	elseif (string.find(this:GetName(), "^AutoBar_Options_ConfigButton_Button")) then
		local fromIndex = AutoBar_Options_ConfigButton.draggingConfigButton;
--		if (fromIndex) then
--			DEFAULT_CHAT_FRAME:AddMessage("AutoBar_Config_Button_OnReceiveDrag " .. fromIndex .. " -> ", 1, 0.5, 0);
--		end
--		DEFAULT_CHAT_FRAME:AddMessage("AutoBar_Config_Button_OnReceiveDrag -> " .. toIndex, 1, 0.5, 0);
		if (fromIndex and fromIndex ~= toIndex) then
			AutoBar_MoveButtonItems(AutoBar_Options_ConfigButton.editting, fromIndex, toIndex);
		elseif (CursorHasItem()) then
			if (dragBag and dragSlot) then
				local button = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
				local name, id = AutoBar.LinkDecode(GetContainerItemLink(dragBag, dragSlot));
				if (type(button) == "table") then
					if (table.getn(button) < toIndex) then
						toIndex = table.getn(button) + 1;	
					end
					AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting][toIndex] = id;
				else
					AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting] = id;
				end
				PutItemInBackpack();
				dragBag = nil;
				dragSlot = nil;
			end
		end                                                          
		AutoBar_Config_Button_Edit(AutoBar_Options_ConfigButton.editting);
	end
	AutoBarConfig.OnShow();
	AutoBar.ConfigUpdated();
	AutoBar_Options_ConfigButton.dragging = nil;
	AutoBar_Options_ConfigButton.draggingConfigButton = nil;
	ResetCursor();
end


function AutoBarConfig.OnEnter()
	if (AutoBar_Options_ConfigButton.dragging) then
		SetCursor("BUY_CURSOR");
	end
end


function AutoBar_Config_Button_ConvertOnClick()
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting];
	local noPopupCheckbox = getglobal("AutoBar_Options_ConfigButton_NoPopup");
	local arrangeOnUseCheckbox = getglobal("AutoBar_Options_ConfigButton_ArrangeOnUse");
--	local noPopupCheckboxText = getglobal("AutoBar_Options_ConfigButton_NoPopupText");
--	local arrangeOnUseCheckboxText = getglobal("AutoBar_Options_ConfigButton_ArrangeOnUseText");
	if (type(buttonInfo) == "table") then
		local tmp = buttonInfo[1];
		AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting] = tmp;
		noPopupCheckbox:Disable();
		arrangeOnUseCheckbox:Disable();
--		noPopupCheckboxText:Disable();
--		arrangeOnUseCheckboxText:Disable();
	else
		local tmp = buttonInfo;
		AutoBar_Config[AutoBar_Player].buttons[AutoBar_Options_ConfigButton.editting] = { tmp };
		noPopupCheckbox:Enable();
		arrangeOnUseCheckbox:Enable();
--		noPopupCheckboxText:Enable();
--		arrangeOnUseCheckboxText:Enable();
	end
	AutoBar_Config_Button_Edit(AutoBar_Options_ConfigButton.editting);
end


function AutoBar_Config_ButtonChooseCategory_OnShow()
	if (AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore and not AutoBar_Category_Info[AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore]) then
		AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore = nil;
	end
	if (AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore) then
		AutoBar_Options_ConfigButton_ChooseCategory_HintText1:Hide();
		local category = AutoBar_Options_ConfigButton_ChooseCategory.categoryexplore;

		FauxScrollFrame_Update(AutoBar_Options_ConfigButton_ChooseCategory_Scroll, table.getn(AutoBar_Category_Info[category].items), 7, 36);
		local offset = FauxScrollFrame_GetOffset(AutoBar_Options_ConfigButton_ChooseCategory_Scroll);
		local index,name,texture;
		for index = 1, 7, 1 do
			local button = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index);
			local hotkey = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index.."HotKey");
			local count  = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index.."Count");
			local icon   = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index.."Icon");
			local text   = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Text"..index);
			button.category = nil;
			if (AutoBar_Category_Info[category].items[index+offset]) then
				if (type(AutoBar_Category_Info[category].items[index+offset]) == "number") then
					name,_,_,_,_,_,_,_,texture = GetItemInfo(AutoBar_Category_Info[category].items[index+offset]);
					if (not name) then
						name = AUTOBAR_CONFIG_NOTFOUND..AutoBar_Category_Info[category].items[index+offset]..")";
						texture = "Interface\\Icons\\INV_Misc_Gift_01";
					elseif (not texture) then
						texture = "Interface\\Icons\\INV_Misc_Gift_02";
					end
				else
					texture = "Interface\\Icons\\INV_Misc_Gift_03";
					name = AutoBar_Category_Info[category].items[index+offset];
				end
				icon:SetTexture(texture);
				text:SetText(name);
				count:SetText("");
				button:Show();
				button.itemid = tonumber(AutoBar_Category_Info[category].items[index+offset]);
			else
				button:Hide();
				button.itemid = nil;
				text:SetText("");
			end
		end
	else
		AutoBar_Options_ConfigButton_ChooseCategory_HintText1:Show();
		local sortedCategories = {};
		for categoryName, rec in AutoBar_Category_Info do
			table.insert(sortedCategories, categoryName);
		end
		table.sort (sortedCategories);
		table.insert(sortedCategories, 1, "EMPTY");
		FauxScrollFrame_Update(AutoBar_Options_ConfigButton_ChooseCategory_Scroll, table.getn(sortedCategories), 7, 36);
		local offset = FauxScrollFrame_GetOffset(AutoBar_Options_ConfigButton_ChooseCategory_Scroll);
		local index;
		for index = 1, 7, 1 do
			local button = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index);
			local hotkey = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index.."HotKey");
			local count  = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index.."Count");
			local icon   = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Button"..index.."Icon");
			local text   = getglobal("AutoBar_Options_ConfigButton_ChooseCategory_Text"..index);
			button.itemid = nil;
			if (sortedCategories[index+offset] == "EMPTY") then
				icon:SetTexture("");
				count:SetText(AUTOBAR_CONFIG_EMPTY);
				text:SetText(AUTOBAR_CONFIG_REMOVECAT);
				button:Show();
				button.category = sortedCategories[index+offset];
			elseif (sortedCategories[index+offset]) then
				icon:SetTexture(AutoBar_GetTexture(sortedCategories[index+offset]));
				count:SetText("");
				text:SetText(AutoBar_Category_Info[sortedCategories[index+offset]].description);
				button.category = sortedCategories[index+offset];
				button:Show();
			else
				button:Hide();
				button.category = nil;
				text:SetText("");
			end
		end
	end
end

function AutoBar_Config_ButtonChooseCategory_OnScroll()
	GameTooltip:Hide();
	AutoBar_Config_ButtonChooseCategory_OnShow();
end

function AutoBar_Config_Button_InsertCustomItem()
	StaticPopup_Show("AUTOBAR_ITEMENTRY");
end


function AutoBar_Options_Bar_Sliders(calledfromslider)
	if (not AutoBar_Config[AutoBar_Player].display) then
		AutoBar_Config[AutoBar_Player].display = {}; 
	end
	--
	local rows, columns, gapping, alpha, width, height, dockshiftx, dockshifty;
	if (calledfromslider) then
		rows    = AutoBar_Options_Bar_Rows:GetValue();
		columns = AutoBar_Options_Bar_Columns:GetValue();
		gapping = AutoBar_Options_Bar_Gapping:GetValue();
		alpha   = AutoBar_Options_Bar_Alpha:GetValue();
		width   = AutoBar_Options_Bar_ButtonWidth:GetValue();
		height  = AutoBar_Options_Bar_ButtonHeight:GetValue();
		dockshiftx = AutoBar_Options_Bar_DockShiftX:GetValue();
		dockshifty = AutoBar_Options_Bar_DockShiftY:GetValue();
	end
	--
	if ((not rows or rows == 0) and AutoBar_Config[AutoBar_Player].display.rows) then 
		rows = AutoBar_Config[AutoBar_Player].display.rows;
	elseif (not rows or rows == 0) then 
		rows = 1; 
	end
	if ((not columns or columns == 0) and AutoBar_Config[AutoBar_Player].display.columns) then 
		columns = AutoBar_Config[AutoBar_Player].display.columns;
	elseif (not columns or columns == 0) then 
		columns = 18; 
	end
	if ((not gapping or gapping == 0) and AutoBar_Config[AutoBar_Player].display.gapping) then 
		gapping = AutoBar_Config[AutoBar_Player].display.gapping;
	elseif (not gapping or gapping == 0) then 
		gapping = 3; 
	end
	if ((not alpha or alpha == 0) and AutoBar_Config[AutoBar_Player].display.alpha) then 
		alpha = AutoBar_Config[AutoBar_Player].display.alpha*10;
	elseif (not alpha or alpha == 0) then 
		alpha = 10; 
	end
	if ((not width or width == 0) and AutoBar_Config[AutoBar_Player].display.buttonWidth) then 
		width = AutoBar_Config[AutoBar_Player].display.buttonWidth;
	elseif (not width or width == 0) then 
		width = 36; 
	end
	if ((not height or height == 0) and AutoBar_Config[AutoBar_Player].display.buttonHeight) then 
		height = AutoBar_Config[AutoBar_Player].display.buttonHeight;
	elseif (not height or height == 0) then 
		height = 36; 
	end
	if ((not dockshiftx or dockshiftx == 0) and AutoBar_Config[AutoBar_Player].display.dockshiftx) then 
		dockshiftx = AutoBar_Config[AutoBar_Player].display.dockshiftx;
	elseif (not dockshiftx or dockshiftx == 0) then 
		dockshiftx = 0; 
	end
	if ((not dockshifty or dockshifty == 0) and AutoBar_Config[AutoBar_Player].display.dockshifty) then 
		dockshifty = AutoBar_Config[AutoBar_Player].display.dockshifty;
	elseif (not dockshifty or dockshifty == 0) then 
		dockshifty = 0; 
	end
	--
	if (calledfromslider) then
		if (rows * columns > AUTOBAR_MAXBUTTONS) then
			if (this:GetName() == "AutoBar_Options_Bar_Rows") then
				rows = AutoBar_Options_Bar_Rows:GetValue();
				columns = math.floor(AUTOBAR_MAXBUTTONS / rows);
			else
				columns = AutoBar_Options_Bar_Columns:GetValue();
				rows = math.floor(AUTOBAR_MAXBUTTONS / columns);
			end
		end
		if (height ~= width and not AutoBar_Config[AutoBar_Player].display.unlockbuttonratio) then
			if (this:GetName() == "AutoBar_Options_Bar_ButtonWidth") then
				height = width;
			elseif (this:GetName() == "AutoBar_Options_Bar_ButtonHeight") then
				width = height;
			end
		end
		--
		AutoBar_Config[AutoBar_Player].display.rows = rows;
		AutoBar_Config[AutoBar_Player].display.columns = columns;
		AutoBar_Config[AutoBar_Player].display.gapping = gapping;
		AutoBar_Config[AutoBar_Player].display.alpha = math.floor(alpha)/10;
		AutoBar_Config[AutoBar_Player].display.buttonWidth = width;
		AutoBar_Config[AutoBar_Player].display.buttonHeight = height;
		AutoBar_Config[AutoBar_Player].display.dockshiftx = dockshiftx;
		AutoBar_Config[AutoBar_Player].display.dockshifty = dockshifty;
	end
	--
	AutoBar_Options_Bar_RowsText:SetText(AUTOBAR_CONFIG_ROW.." - "..rows);
	AutoBar_Options_Bar_Rows:SetValue(rows);
	AutoBar_Options_Bar_ColumnsText:SetText(AUTOBAR_CONFIG_COLUMN.." - "..columns);
	AutoBar_Options_Bar_Columns:SetValue(columns);
	AutoBar_Options_Bar_GappingText:SetText(AUTOBAR_CONFIG_GAPPING.." - "..gapping);
	AutoBar_Options_Bar_Gapping:SetValue(gapping);
	AutoBar_Options_Bar_AlphaText:SetText(AUTOBAR_CONFIG_ALPHA.." - "..math.floor(alpha)/10);
	AutoBar_Options_Bar_Alpha:SetValue(alpha);
	AutoBar_Options_Bar_ButtonWidthText:SetText(AUTOBAR_CONFIG_BUTTONWIDTH.." - "..width);
	AutoBar_Options_Bar_ButtonWidth:SetValue(width);
	AutoBar_Options_Bar_ButtonHeightText:SetText(AUTOBAR_CONFIG_BUTTONHEIGHT.." - "..height);
	AutoBar_Options_Bar_ButtonHeight:SetValue(height);
	AutoBar_Options_Bar_DockShiftXText:SetText(AUTOBAR_CONFIG_DOCKSHIFTX.." - "..dockshiftx);
	AutoBar_Options_Bar_DockShiftX:SetValue(dockshiftx);
	AutoBar_Options_Bar_DockShiftYText:SetText(AUTOBAR_CONFIG_DOCKSHIFTY.." - "..dockshifty);
	AutoBar_Options_Bar_DockShiftY:SetValue(dockshifty);

	AutoBar_SetupVisual();
end

function AutoBar_Options_CheckBox_Setup()
	AutoBar_Options_Bar_DockingMainBarText:SetText(AUTOBAR_CONFIG_DOCKING_MAINMENU);
	AutoBar_Options_Bar_SmartSelfCastText:SetText(AUTOBAR_CONFIG_SMARTSELFCAST);
	AutoBar_Options_Bar_WidthHeightLockedText:SetText(AUTOBAR_CONFIG_WIDTHHEIGHTLOCKED);
	AutoBar_Options_Bar_ReverseButtonsText:SetText(AUTOBAR_CONFIG_REVERSE);
	AutoBar_Options_Bar_HideKeyTextText:SetText(AUTOBAR_CONFIG_HIDEKEYBINDING);
	AutoBar_Options_Bar_HideCountText:SetText(AUTOBAR_CONFIG_HIDECOUNT);
	AutoBar_Options_Bar_ShowEmptyButtonsText:SetText(AUTOBAR_CONFIG_SHOWEMPTY);
	AutoBar_Options_Bar_ShowCategoryIconText:SetText(AUTOBAR_CONFIG_SHOWCATEGORYICON);
	AutoBar_Options_Bar_HideTooltipsText:SetText(AUTOBAR_CONFIG_HIDETOOLTIP);
	AutoBar_Options_Bar_PopupToTopText:SetText(nil);
	AutoBar_Options_Bar_PopupToTopText:Hide();
	AutoBar_Options_Bar_PopupToLeftText:SetText(nil); 
	AutoBar_Options_Bar_PopupToLeftText:Hide();
	AutoBar_Options_Bar_PopupToRightText:SetText(AUTOBAR_CONFIG_POPUPDIRECTION);
	AutoBar_Options_Bar_PopupToBottomText:SetText(nil);
	AutoBar_Options_Bar_PopupToBottomText:Hide();
	AutoBar_Options_Bar_PopupDisableText:SetText(AUTOBAR_CONFIG_POPUPDISABLE);
	AutoBar_Options_Bar_PopupOnShiftText:SetText(AUTOBAR_CONFIG_POPUPONSHIFT);
	AutoBar_Options_Bar_HideDragHandleText:SetText(AUTOBAR_CONFIG_HIDEDRAGHANDLE);
	AutoBar_Options_Bar_PlainButtonsText:SetText(AUTOBAR_CONFIG_PLAINBUTTONS);

--	AutoBar_Options_Bar_PopupToTop:SetHitRectInsets(0, 0, 0, 0);
--	AutoBar_Options_Bar_PopupToLeft:SetHitRectInsets(0, 0, 0, 0);
--	AutoBar_Options_Bar_PopupToBottom:SetHitRectInsets(0, 0, 0, 0);

	if (AutoBar_Config[AutoBar_Player].display.docking == "MAINMENU") then
		AutoBar_Options_Bar_DockingMainBar:SetChecked(1);
	else
		AutoBar_Options_Bar_DockingMainBar:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].autoSmartSelfCast) then
		AutoBar_Options_Bar_SmartSelfCast:SetChecked(1);
	else
		AutoBar_Options_Bar_SmartSelfCast:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.unlockbuttonratio) then
		AutoBar_Options_Bar_WidthHeightLocked:SetChecked(0);
	else
		AutoBar_Options_Bar_WidthHeightLocked:SetChecked(1);
	end
	if (AutoBar_Config[AutoBar_Player].display.reversebuttons) then
		AutoBar_Options_Bar_ReverseButtons:SetChecked(1);
	else
		AutoBar_Options_Bar_ReverseButtons:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.hidekeytext) then
		AutoBar_Options_Bar_HideKeyText:SetChecked(1);
	else
		AutoBar_Options_Bar_HideKeyText:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.hidecount) then
		AutoBar_Options_Bar_HideCount:SetChecked(1);
	else
		AutoBar_Options_Bar_HideCount:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.showemptybuttons) then
		AutoBar_Options_Bar_ShowEmptyButtons:SetChecked(1);
	else
		AutoBar_Options_Bar_ShowEmptyButtons:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.showCategoryIcon) then
		AutoBar_Options_Bar_ShowCategoryIcon:SetChecked(1);
	else
		AutoBar_Options_Bar_ShowCategoryIcon:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.hidetooltips) then
		AutoBar_Options_Bar_HideTooltips:SetChecked(1);
	else
		AutoBar_Options_Bar_HideTooltips:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.popupToTop) then
		AutoBar_Options_Bar_PopupToTop:SetChecked(1);
	else
		AutoBar_Options_Bar_PopupToTop:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.popupToLeft) then
		AutoBar_Options_Bar_PopupToLeft:SetChecked(1);
	else
		AutoBar_Options_Bar_PopupToLeft:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.popupToRight) then
		AutoBar_Options_Bar_PopupToRight:SetChecked(1);
	else
		AutoBar_Options_Bar_PopupToRight:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.popupToBottom) then
		AutoBar_Options_Bar_PopupToBottom:SetChecked(1);
	else
		AutoBar_Options_Bar_PopupToBottom:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.popupDisable) then
		AutoBar_Options_Bar_PopupDisable:SetChecked(1);
	else
		AutoBar_Options_Bar_PopupDisable:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.popupOnShift) then
		AutoBar_Options_Bar_PopupOnShift:SetChecked(1);
	else
		AutoBar_Options_Bar_PopupOnShift:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.hideDragHandle) then
		AutoBar_Options_Bar_HideDragHandle:SetChecked(1);
	else
		AutoBar_Options_Bar_HideDragHandle:SetChecked(0);
	end
	if (AutoBar_Config[AutoBar_Player].display.plainButtons) then
		AutoBar_Options_Bar_PlainButtons:SetChecked(1);
	else
		AutoBar_Options_Bar_PlainButtons:SetChecked(0);
	end
end


function AutoBarConfig:CheckBoxOnClick()
end


function AutoBar_Options_CheckBox_OnCheck()
	local button = this:GetName(); 
	if (this:GetChecked()) then
		if (button == "AutoBar_Options_Bar_DockingMainBar") then
			AutoBar_Config[AutoBar_Player].display.docking = "MAINMENU";
		elseif (button == "AutoBar_Options_Bar_SmartSelfCast") then
			AutoBar_Config[AutoBar_Player].autoSmartSelfCast = true;
		elseif (button == "AutoBar_Options_Bar_WidthHeightLocked") then
			AutoBar_Config[AutoBar_Player].display.unlockbuttonratio = nil;
		elseif (button == "AutoBar_Options_Bar_ReverseButtons") then
			AutoBar_Config[AutoBar_Player].display.reversebuttons = 1;
		elseif (button == "AutoBar_Options_Bar_HideKeyText") then
			AutoBar_Config[AutoBar_Player].display.hidekeytext = 1;
		elseif (button == "AutoBar_Options_Bar_HideCount") then
			AutoBar_Config[AutoBar_Player].display.hidecount = 1;
		elseif (button == "AutoBar_Options_Bar_ShowEmptyButtons") then
			AutoBar_Config[AutoBar_Player].display.showemptybuttons = 1;
		elseif (button == "AutoBar_Options_Bar_ShowCategoryIcon") then
			AutoBar_Config[AutoBar_Player].display.showCategoryIcon = 1;
		elseif (button == "AutoBar_Options_Bar_HideTooltips") then
			AutoBar_Config[AutoBar_Player].display.hidetooltips = 1;
		elseif (button == "AutoBar_Options_Bar_PopupToTop") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = 1;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = nil;
			AutoBar_Config[AutoBar_Player].display.popupToRight = nil;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = nil;
		elseif (button == "AutoBar_Options_Bar_PopupToLeft") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = nil;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = 1;
			AutoBar_Config[AutoBar_Player].display.popupToRight = nil;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = nil;
		elseif (button == "AutoBar_Options_Bar_PopupToRight") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = nil;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = nil;
			AutoBar_Config[AutoBar_Player].display.popupToRight = 1;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = nil;
		elseif (button == "AutoBar_Options_Bar_PopupToBottom") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = nil;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = nil;
			AutoBar_Config[AutoBar_Player].display.popupToRight = nil;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = 1;
		elseif (button == "AutoBar_Options_Bar_PopupDisable") then
			AutoBar_Config[AutoBar_Player].display.popupDisable = 1;
		elseif (button == "AutoBar_Options_Bar_PopupOnShift") then
			AutoBar_Config[AutoBar_Player].display.popupOnShift = 1;
		elseif (button == "AutoBar_Options_Bar_HideDragHandle") then
			AutoBar_Config[AutoBar_Player].display.hideDragHandle = 1;
		elseif (button == "AutoBar_Options_Bar_PlainButtons") then
			AutoBar_Config[AutoBar_Player].display.plainButtons = 1;
		end
	else
		if (button == "AutoBar_Options_Bar_DockingMainBar") then
			AutoBar_Config[AutoBar_Player].display.docking = nil;
		elseif (button == "AutoBar_Options_Bar_SmartSelfCast") then
			AutoBar_Config[AutoBar_Player].autoSmartSelfCast = nil;
		elseif (button == "AutoBar_Options_Bar_WidthHeightLocked") then
			AutoBar_Config[AutoBar_Player].display.unlockbuttonratio = 1;
		elseif (button == "AutoBar_Options_Bar_ReverseButtons") then
			AutoBar_Config[AutoBar_Player].display.reversebuttons = nil;
		elseif (button == "AutoBar_Options_Bar_HideKeyText") then
			AutoBar_Config[AutoBar_Player].display.hidekeytext = nil;
		elseif (button == "AutoBar_Options_Bar_HideCount") then
			AutoBar_Config[AutoBar_Player].display.hidecount = nil;
		elseif (button == "AutoBar_Options_Bar_ShowEmptyButtons") then
			AutoBar_Config[AutoBar_Player].display.showemptybuttons = nil;
		elseif (button == "AutoBar_Options_Bar_ShowCategoryIcon") then
			AutoBar_Config[AutoBar_Player].display.showCategoryIcon = nil;
		elseif (button == "AutoBar_Options_Bar_HideTooltips") then
			AutoBar_Config[AutoBar_Player].display.hidetooltips = nil;
		elseif (button == "AutoBar_Options_Bar_PopupDisable") then
			AutoBar_Config[AutoBar_Player].display.popupDisable = nil;
		elseif (button == "AutoBar_Options_Bar_PopupOnShift") then
			AutoBar_Config[AutoBar_Player].display.popupOnShift = nil;
		elseif (button == "AutoBar_Options_Bar_HideDragHandle") then
			AutoBar_Config[AutoBar_Player].display.hideDragHandle = nil;
		elseif (button == "AutoBar_Options_Bar_PlainButtons") then
			AutoBar_Config[AutoBar_Player].display.plainButtons = nil;
		end
	end
	AutoBar_Options_CheckBox_Setup();
	AutoBar_SetupVisual();
end


-- Individual smart self cast checkbox handling
function AutoBar_Options_ConfigButton_Option_OnCheck()
	local slotIndex = AutoBar_Options_ConfigButton.editting; 
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[slotIndex];
	local category;
	if (type(buttonInfo) == "table") then
		category = buttonInfo[this:GetID()];
	else
		category = buttonInfo;
	end
	if (not AutoBar_Config[AutoBar_Player].smartselfcast) then
		AutoBar_Config[AutoBar_Player].smartselfcast = {};
	end
	AutoBar_Config[AutoBar_Player].smartselfcast[category] = this:GetChecked();
	AutoBar_Config_Button_Edit();
end


AutoBarConfigSlot = {};
function AutoBarConfigSlot:Setup()
	getglobal("AutoBar_Options_ConfigButton_NoPopupText"):SetText(AUTOBAR_CONFIG_NOPOPUP);
	getglobal("AutoBar_Options_ConfigButton_ArrangeOnUseText"):SetText(AUTOBAR_CONFIG_ARRANGEONUSE);
--	getglobal("AutoBar_Options_ConfigButton_NoPopup"):SetDisabledTextColor(.5, .5, .5, .5);
--	getglobal("AutoBar_Options_ConfigButton_ArrangeOnUse"):SetDisabledTextColor(.5, .5, .5, .5);
end


function AutoBarConfigSlot:OnCheck()
	local slotIndex = AutoBar_Options_ConfigButton.editting; 
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[slotIndex];
	local buttonName = this:GetName(); 

	if (type(buttonInfo) == "table") then
		if (buttonName == "AutoBar_Options_ConfigButton_NoPopup") then
			buttonInfo.noPopup = this:GetChecked();
		elseif (buttonName == "AutoBar_Options_ConfigButton_ArrangeOnUse") then
			buttonInfo.arrangeOnUse = this:GetChecked();
		end
	end
	
	AutoBar_Config_Button_Edit();
end


function AutoBar_Options_MoveArrow_OnClick(fromIndex, direction)
	local button = AutoBar_Options_ConfigButton.editting;
	if (type(AutoBar_Config[AutoBar_Player].buttons[button]) == "table") then
		local toIndex;
		if (direction == "UP") then
			toIndex = fromIndex - 1;
		elseif (direction == "DOWN") then
			toIndex = fromIndex + 1;
		end
		AutoBar_MoveButtonItems(button, fromIndex, toIndex);
		AutoBar_Config_Button_Edit(button);
	end
end


function AutoBar_MoveButtonItems(button, fromIndex, toIndex)
	if (type(AutoBar_Config[AutoBar_Player].buttons[button]) == "table") then
		if (not AutoBar_Config[AutoBar_Player].buttons[button][fromIndex] or fromIndex == toIndex) then
			-- Move to self so do nothing
			return;
		end
		if (not AutoBar_Config[AutoBar_Player].buttons[button][fromIndex]) then
			-- Dont allow swapping empties into the list
			return;
		end
		local temp = AutoBar_Config[AutoBar_Player].buttons[button][fromIndex];
		if (not AutoBar_Config[AutoBar_Player].buttons[button][toIndex]) then
			-- Move to the end
			table.remove(AutoBar_Config[AutoBar_Player].buttons[button], fromIndex);
			table.insert(AutoBar_Config[AutoBar_Player].buttons[button], temp);
		else
			-- Swap the two
			AutoBar_Config[AutoBar_Player].buttons[button][fromIndex] = AutoBar_Config[AutoBar_Player].buttons[button][toIndex];
			AutoBar_Config[AutoBar_Player].buttons[button][toIndex] = temp;
		end
	end
end
